package com.ruoyi.bst.device.utils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

import com.ruoyi.bst.device.domain.Device;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceLocationType;
import com.ruoyi.bst.device.domain.vo.DeviceCacheInfo;
import com.ruoyi.bst.orderDevice.domain.enums.OrderDeviceStatus;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.map.GpsCoordinateUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.IotDeviceSysInfo;
import com.ruoyi.iot.util.BatUtil;

public class DeviceUtil {

    public static void setIotSysInfo(DeviceVO device, IotDeviceSysInfo sys, LocalDateTime at, boolean setNull) {
        setIotSysInfo(device, sys, at, setNull, DeviceLocationType.DEVICE);
    }

    /**
     * 设置设备系统信息
     *
     * @param device          设备
     * @param sys             设备信息
     * @param at              时间
     * @param setNull         若定位不正常，是否设置为空
     * @param convertLocation 是否转换经纬度坐标系
     */
    public static void setIotSysInfo(DeviceVO device, IotDeviceSysInfo sys, LocalDateTime at, boolean setNull,
            DeviceLocationType locationType) {
        if (device == null || sys == null) {
            return;
        }

        // 转换经纬度坐标系并赋值
        // 只有定位是正常的才认为是有获取到定位
        if (DeviceUtil.validLocation(sys.getLon(), sys.getLat())) {
            List<BigDecimal> coordinates = null;

            // 转换坐标
            if (locationType == DeviceLocationType.DEVICE) {
                coordinates = GpsCoordinateUtils.coordinateConvert(sys.getLon(), sys.getLat());
            } else if (locationType == DeviceLocationType.PHONE) {
                coordinates = Arrays.asList(sys.getLon(), sys.getLat());
            }

            if (coordinates != null && coordinates.size() >= 2) {
                device.setLongitude(coordinates.get(0));
                device.setLatitude(coordinates.get(1));
                device.setLastLocationTime(at);
                device.setLocationType(locationType.getCode());
            }
        } else if (setNull) {
            device.setLongitude(null);
            device.setLatitude(null);
            device.setLastLocationTime(null);
            device.setLocationType(null);
        }

        device.setLastTime(at);
        device.setVoltage(sys.getBat());
        device.setSignalStrength(sys.getCsq());
        device.setQuality(sys.getQ());
        device.setSatellites(sys.getS());
        device.setIotStatus(sys.getStatus());
        device.setRemainingPower(
                BatUtil.calcPowerPoint(device.getVoltage(), device.getModelFullVoltage(), device.getModelLowVoltage()));
    }

    /**
     * 设置设备信息
     *
     * @param device 设备
     * @param iot    设备信息
     */
    public static void setIotInfo(DeviceVO device, IotDeviceInfo iot) {
        if (device == null || iot == null) {
            return;
        }

        IotDeviceSysInfo sys = iot.getSys();
        if (sys != null) {
            setIotSysInfo(device, sys, iot.getAt(), false);
        }
        device.setSoftwareVersion(iot.getVer());
    }

    /**
     * 设置设备信息
     *
     * @param device 设备
     * @param info   设备信息
     */
    public static void setIotInfo(Device device, DeviceCacheInfo info) {
        if (device == null || info == null) {
            return;
        }
        if (info.getVoltage() != null) {
            device.setVoltage(info.getVoltage());
        }
        if (info.getSignalStrength() != null) {
            device.setSignalStrength(info.getSignalStrength());
        }
        if (info.getQuality() != null) {
            device.setQuality(info.getQuality());
        }
        if (info.getSatellites() != null) {
            device.setSatellites(info.getSatellites());
        }
        if (info.getIotStatus() != null) {
            device.setIotStatus(info.getIotStatus());
        }
        if (info.getLongitude() != null) {
            device.setLongitude(info.getLongitude());
        }
        if (info.getLatitude() != null) {
            device.setLatitude(info.getLatitude());
        }
        if (info.getRemainingPower() != null) {
            device.setRemainingPower(info.getRemainingPower());
        }
        if (info.getLastTime() != null) {
            device.setLastTime(info.getLastTime());
        }
        if (info.getLastLocationTime() != null) {
            device.setLastLocationTime(info.getLastLocationTime());
        }
        if (info.getSoftwareVersion() != null) {
            device.setSoftwareVersion(info.getSoftwareVersion());
        }

        // 在线状态
        if (info.getOnlineStatus() != null) {
            device.setOnlineStatus(info.getOnlineStatus());
        }
        if (info.getLastOnlineTime() != null) {
            device.setLastOnlineTime(info.getLastOnlineTime());
        }
    }

    /**
     * 判断设备是否有订单正在使用
     *
     * @param device 设备
     * @return 是否正在使用
     */
    public static boolean isOrderUsing(DeviceVO device) {
        return device != null && device.getOrderDeviceId() != null
                && OrderDeviceStatus.inUse().contains(device.getOrderDeviceStatus());
    }

    /**
     * 判断经纬度是否有效
     *
     * @param lon 经度
     * @param lat 纬度
     * @return 是否有效
     */
    public static boolean validLocation(BigDecimal lon, BigDecimal lat) {
        return lon != null && lat != null
                && !MathUtils.equals(lon, BigDecimal.ZERO)
                && !MathUtils.equals(lat, BigDecimal.ZERO);
    }

    /**
     * 判断设备信号是否弱
     *
     * @param device 设备
     * @return 是否弱
     */
    public static boolean isLowSignal(DeviceVO device) {
        return device != null && device.getSignalStrength() != null && device.getSignalStrength() < 10;
    }

    /**
     * 判断卫星信号是否弱
     *
     * @param device 设备
     * @return 是否弱
     */
    public static boolean isLowSatelliteSignal(DeviceVO device) {
        return device != null && device.getSatellites() != null && device.getSatellites() <= 5;
    }
}